/*
 * Bitmap.c
 *
 *  Created on: 2009/07/07
 *      Author: ashida
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Bitmap.h>
#include <Common.h>

/* wb_ǂ */
static bool _readHeader(Bitmap *bitmap, FILE *fp);
/* Recǂ */
static bool _readContents(Bitmap *bitmap, FILE *fp);
/* t@C^Cvǂ */
static bool _readFileType(FILE *fp);
/* Rgǂ */
static bool _readComment(FILE *fp);
/* 𑜓xǂ */
static bool _readResolution(unsigned int *width, unsigned int *height, FILE *fp);
/* Xy[XC^uCsR[hȊO̕1擾 */
static char _getChar(FILE *fp);
/* ̊m */
static void _allocMemory(Bitmap *bitmap);

/*------------------------------------------------------------------------------------------*/
/* rbg}bv\̂̏ */
void initBitmap(Bitmap *bitmap, unsigned int width, unsigned int height){

	/* ݒ */
	bitmap->width = width;
	/* ݒ */
	bitmap->height = height;

	/* ̊m */
	_allocMemory(bitmap);

}

/*------------------------------------------------------------------------------------------*/
/* rbg}bvɉfݒ肷 */
void setPixel(Bitmap *bitmap, unsigned int x, unsigned int y, Pixel p){
	bitmap->data[y][x] = p;
}
/*------------------------------------------------------------------------------------------*/
/* rbg}bvɕ`悳Ăf𓾂 */
Pixel getPixel(Bitmap *bitmap, unsigned int x, unsigned int y){
	return bitmap->data[y][x];
}
/*------------------------------------------------------------------------------------------*/
/* rbg}bv */
void freeBitmap(Bitmap *bitmap){
	/* ϐ */
	int i;
	/* ̉ */
	for(i=0; i<bitmap->height; i++){
		free(bitmap->data[i]);
	}
	free(bitmap->data);
}
/*------------------------------------------------------------------------------------------*/
/* t@C摜ǂݍ */
bool readBitmap(Bitmap *bitmap, char *filename){
	/* t@C|C^ */
	FILE *fp;
	/* ߂l */
	bool rv = TRUE;

	/* t@CI[v */
	if((fp = fopen(filename, "r")) == NULL){
		fprintf(stderr, "Can't open file : %s\n", filename);
		exit(1);
	}

	/* wb_ǂ */
	rv &= _readHeader(bitmap, fp);
	/* Recǂ */
	rv &= _readContents(bitmap, fp);

	/* t@CN[Y */
	fclose(fp);

	return rv;
}
/*------------------------------------------------------------------------------------------*/
/************************** */
/* X^eBbN֐ */
/************************** */
/* wb_ǂ */
static bool _readHeader(Bitmap *bitmap, FILE *fp){
	/* ߂l */
	bool rv = TRUE;

	/* t@C^Cvǂ */
	rv &= _readFileType(fp);
	/* Rgǂ */
	rv &= _readComment(fp);
	/* 𑜓xǂ */
	rv &= _readResolution(&(bitmap->width), &(bitmap->height), fp);

	/* ̊mۂs */
	_allocMemory(bitmap);
	return rv;
}

/*------------------------------------------------------------------------------------------*/
/* Recǂ */
static bool _readContents(Bitmap *bitmap, FILE *fp){
	/* ϐ */
	unsigned int i,j;
	/* t@Cǂݍ񂾕 */
	char ch;

	for(j=0; j<bitmap->height; j++){
		for(i=0; i<bitmap->width; i++){
			/* Xy[XC^uCsR[hȊO̕1擾 */
			ch = _getChar(fp);
			/* t@C1擾 */
			if(ch == EOF){
				return FALSE;
			}
			/* ǂݍ񂾕f\ł */
			if(ch == BLACK_CHAR){
				bitmap->data[j][i] = BLACK;
			}
			/* ǂݍ񂾕f\ł */
			else if(ch == WHITE_CHAR){
				bitmap->data[j][i] = WHITE;
			}
			/* zO */
			else{
				return FALSE;
			}
		}
	}

	return TRUE;

}
/*------------------------------------------------------------------------------------------*/
/* t@C^Cvǂ */
static bool _readFileType(FILE *fp){
	/* obt@ */
	char buffer[BUFFER_SIZE];
	/* t@C1s擾 */
	if((fgets(buffer, BUFFER_SIZE, fp)) == NULL){
		return FALSE;
	}
	/* sR[h */
	buffer[strlen(buffer)-1] = '\0';
	/* PBM̃AXL[łȂ */
	if(strcmp(buffer, PBM_ASCII)){
		return FALSE;
	}
	return TRUE;
}
/*------------------------------------------------------------------------------------------*/
/* Rgǂ */
static bool _readComment(FILE *fp){
	/* obt@ */
	char buffer[BUFFER_SIZE];
	/* t@C1s擾 */
	if((fgets(buffer, BUFFER_SIZE, fp)) == NULL){
		return FALSE;
	}
	/* ǂݍ񂾕񂪃RgłȂ */
	if(buffer[0] != COMMENT_PREFIX){
		return FALSE;
	}
	return TRUE;

}
/*------------------------------------------------------------------------------------------*/
/* 𑜓xǂ */
static bool _readResolution(unsigned int *width, unsigned int *height, FILE *fp){
	/* obt@ */
	char buffer[BUFFER_SIZE];
	/* t@C1s擾 */
	if((fgets(buffer, BUFFER_SIZE, fp)) == NULL){
		return FALSE;
	}
	/* ƍĂȂ */
	if(sscanf(buffer, "%d %d", width, height) != 2){
		return FALSE;
	}

	return TRUE;
}
/*------------------------------------------------------------------------------------------*/
/* Xy[XC^uCsR[hȊO̕1擾 */
static char _getChar(FILE *fp){
	/* t@C擾1 */
	char ch;

	do{
		ch = fgetc(fp);
	}while(ch == ' ' || ch == '\t' || ch == '\n');

	return ch;
}
/*------------------------------------------------------------------------------------------*/
/* ̊m */
static void _allocMemory(Bitmap *bitmap){
	/* ϐ */
	unsigned int i,j;

	bitmap->data = (unsigned char **)malloc(sizeof(unsigned char *)*bitmap->height);
	for(i=0; i<bitmap->height; i++){
		bitmap->data[i] = (unsigned char *)malloc(sizeof(unsigned char)*bitmap->width);
		/* Fɏ */
		for(j=0; j<bitmap->width; j++){
			bitmap->data[i][j] = WHITE;
		}
	}

}
/*------------------------------------------------------------------------------------------*/
