/*
 * Feature.h
 *
 *  Created on: 2009/07/07
 *      Author: ashida
 */

#ifndef FEATURE_H_
#define FEATURE_H_

#include <stdio.h>
#include "Mesh.h"
#include "Bitmap.h"
#include "Dictionary.h"

/** メッシュ特徴量を得る
 *	@param mesh メッシュ特徴量
 *	@param bitmap ビットマップ
 */
extern void calcMeshFeature(Mesh *mesh, Bitmap *bitmap);

/** メッシュ特徴量を表示する
 * @param mesh メッシュ特徴量
 */
extern void printMeshFeature(Mesh *mesh);

/** 初期化する
 * 	@param mesh 初期化するメッシュ特徴量
 */
extern void initMeshFeature(Mesh *mesh);

/** メッシュ特徴量を加えていく
 * @param mesh 加えられるメッシュ特徴量
 * @param sum 加えられた結果
 */
extern void addMeshFeature(Mesh *mesh, Mesh *sum);

/** 各次元を割る
 * 	@param mesh メッシュ特徴量
 * 	@param num 割る数
 */
extern void divideMesh(Mesh *mesh, unsigned int num);

/** 1次元の配列として特徴量を得る
 * 	@param mesh メッシュ特徴量
 * 	@return 1次元にしたメッシュ特徴量
 */
extern unsigned int *getSerialFeatureMesh(Mesh *mesh);

/** 1次元の配列として表現された特徴量をメッシュ特徴量用（2次元配列）に設定する
 * 	@param mesh メッシュ特徴量
 * 	@param serial_feature 1次元であらわした特徴量
 */
extern void setSerialFeatureMesh(Mesh *mesh, unsigned int *serial_feature);

/** メッシュ特徴量の次元数を得る
 * 	@return 次元数
 */
extern unsigned int getMeshDimension(void);

/** 距離を得る
 * 	@param mesh1 メッシュ特徴量１
 * 	@param mesh2 メッシュ特徴量2
 */
extern unsigned int getDistanceMesh(Mesh *mesh1, Mesh *mesh2);

/** ビットマップ構造体の初期化
 * 	@param bitmap 初期化するビットマップ構造体
 * 	@param width 幅
 * 	@param height 高さ
 */
extern void initBitmap(Bitmap *bitmap, unsigned int width, unsigned int height);

/** ビットマップに画素を設定する
 * 	@param bitmap	ビットマップ
 *	@param x		X座標
 *	@param y		Y座標
 *	@param pixel	画素
 */
extern void setPixel(Bitmap *bitmap, unsigned int x, unsigned int y, Pixel pixel );

/** ビットマップに描画されている画素を得る
 *	@param bitmap 	ビットマップ
 *	@param x		X座標
 *	@param y		Y座標
 *	@return			画素
 */
extern Pixel getPixel(Bitmap *bitmap, unsigned int x, unsigned int y);

/** ビットマップを解放する
 *	@param bitmap 消去するビットマップ
 */
extern void freeBitmap(Bitmap *bitmap);

/** ファイルより画像を読み込む
 *	@param bitmap ビットマップ
 *	@param filename ファイル名
 *	@return 読み込みに成功すると真が返る
 */
extern bool readBitmap(Bitmap *bitmap, char *filename);


/** 文字コードを設定する
 * 	@param element 辞書の要素
 * 	@param code 文字コード
 */
extern void setCode(DictionaryElement *element, unsigned int code);

/** 特徴量を設定する
 * 	@param element 辞書の要素
 * 	@param feature 特徴量
 */
extern void setFeature(DictionaryElement *element, void *feature);

/** 辞書要素のメモリを確保する
 * 	@return 辞書要素
 */
extern DictionaryElement *allocDictionaryElement(void);

/** 辞書要素のためのメモリ解放
 * 	@param element 辞書要素
 */
extern void freeDictionaryElement(DictionaryElement *element);

/** 辞書要素を保存する
 * 	@param element 辞書要素
 * 	@param dimension 次元数
 * 	@param fp ファイルポインタ
 */
extern void writeDictionaryElement(DictionaryElement *element, unsigned int dimension, FILE *fp);

/** 辞書要素を読み込む
 * 	@param element 辞書要素
 * 	@param dimension 次元数
 * 	@param fp ファイルポインタ
 */
extern void readDictionaryElement(DictionaryElement *element, unsigned int dimension, FILE *fp);

/** 辞書を初期化する
 * 	@param dictionary 辞書
 * 	@param dimension 次元数
 */
extern void initDictionary(Dictionary *dictionary, unsigned int dimension);

/** 要素数を設定する
 * 	@param dictionary 辞書
 * 	@param element_num 要素数
 */
extern void setElementNum(Dictionary *dictionary, unsigned int element_num);

/** 辞書に追加する
 * 	@param dictionary 辞書
 * 	@param code 文字コード
 * 	@param feature 特徴量
 */
extern void addElement(Dictionary *dictionary, unsigned int code, unsigned int *feature);

/** 辞書のメモリの解放
 * 	@param dictionary 辞書
 */
extern void freeDictionary(Dictionary *dictionary);

/** 辞書を保存する
 * 	@param dictionary 辞書
 * 	@param filename ファイル名
 */
extern void writeDictionary(Dictionary *dictionary, char *filename);

/** 辞書を読み込む
 * 	@param dictionary 辞書
 * 	@param filename ファイル名
 */
extern void readDictionary(Dictionary *dictionary, char *filename);

/** 指定された特徴量を得る
 * 	@param dictionary 辞書
 * 	@param id 通し番号
 * 	@return 特徴量
 */
extern unsigned int *getFeature(Dictionary *dictionary, int id);

/** 登録されている要素数を得る
 * 	@param dictionary 辞書
 * 	@return 登録されている要素数
 */
extern unsigned int getElementNum(Dictionary *dictionary);

/** 文字コードを得る
 * 	@param dictionary 辞書
 * 	@param id 通し番号
 */
extern unsigned int getCode(Dictionary *dictionary, unsigned int id);

#endif /* FEATURE_H_ */
